@@ -6,12 +6,15 @@ if(.AzureContainers$docker == "" ||
    .AzureContainers$helm == "")
     skip("Tests skipped: external tools not found")
 
+echo <- getOption("azure_containers_tool_echo")
+options(azure_containers_tool_echo=FALSE)
+
 test_that("Docker works",
 {
     cmd <- "--help"
     obj <- call_docker(cmd)
     expect_is(obj, "list")
-    expect_identical(obj$cmdline, "docker --help")
+    expect_true(grepl("docker --help", obj$cmdline, fixed=TRUE))
 })
 
 test_that("Docker compose works",
@@ -19,7 +22,7 @@ test_that("Docker compose works",
     cmd <- "--help"
     obj <- call_docker_compose(cmd)
     expect_is(obj, "list")
-    expect_identical(obj$cmdline, "docker-compose --help")
+    expect_true(grepl("docker-compose --help", obj$cmdline, fixed=TRUE))
 })
 
 test_that("Kubectl works",
@@ -27,7 +30,7 @@ test_that("Kubectl works",
     cmd <- "--help"
     obj <- call_kubectl(cmd)
     expect_is(obj, "list")
-    expect_identical(trimws(obj$cmdline), "kubectl --help")
+    expect_true(grepl("kubectl --help", obj$cmdline, fixed=TRUE))
 })
 
 test_that("Helm works",
@@ -35,5 +38,9 @@ test_that("Helm works",
     cmd <- "--help"
     obj <- call_helm(cmd)
     expect_is(obj, "list")
-    expect_identical(trimws(obj$cmdline), "helm --help")
+    expect_true(grepl("helm --help", obj$cmdline, fixed=TRUE))
+})
+
+teardown({
+    options(azure_containers_tool_echo=echo)
 })
